/*
        I2C Functions

        Terry Wu,       Oct. 1994.      Created.
*/

#include "BT878.c"

#define READ_MAX_RETRIES     3
#define READ_MAX_RETRIES_1   3
#define READ_MAX_RETRIES_2   3
#define WRITE_MAX_RETRIES    10

/*
        Internal Functions
*/
void _I2C_START(void)
{
    HWI2C_SetSDA(1);
    HWI2C_SetSCL(1);
    HWI2C_SetSDA(0);
    HWI2C_SetSCL(0);
}

void _I2C_STOP(void)
{
    HWI2C_SetSDA(0);
    HWI2C_SetSCL(1);
    HWI2C_SetSDA(1);
}

int _I2C_ACKNOWLEDGE(void)
{
    int Status;

    HWI2C_SetSDA(1);
    HWI2C_SetSCL(1);
    Status = HWI2C_GetSDA();
    HWI2C_SetSCL(0);

    if (Status != 0)
        return 0;
    else
        return 1;
}

int _I2C_SENDACKNOWLEDGE(void)
{
    HWI2C_SetSDA(1);
    HWI2C_SetSDA(0);
    HWI2C_SetSCL(1);
    HWI2C_SetSCL(0);
    HWI2C_SetSDA(1);

    return 1;
}

int _I2C_ReadData(void)
{
    int      Status;
    int      idx;
    int      Data = 0;

    HWI2C_SetSCL(0);
    for (idx=0;idx<8;idx++)
    {
        HWI2C_SetSDA(1);
        HWI2C_SetSCL(1);
        if (HWI2C_GetSDA()) Data |= 1;
        HWI2C_SetSCL(0);
        Data <<= 1;
        if (HWI2C_IsErrorOccur())
        {
            return 0;
        }
    }

    if (!(Status = _I2C_SENDACKNOWLEDGE()))
    {
#ifdef DEBUG
        printf("No data reading acknowledge.                      \n");
#endif
    }

    return Data >> 1;
}

int _I2C_WriteData(int  Data)
{
    int      idx;

    for (idx=0;idx<8;idx++)
    {
        if (Data & 0x80)
            HWI2C_SetSDA(1);
        else
            HWI2C_SetSDA(0);

        HWI2C_SetSCL(1);
        HWI2C_SetSCL(0);

        Data <<= 1;
    }

    return _I2C_ACKNOWLEDGE();
}

void _I2C_Initial(void)
{
    I2C_WAIT = 1L;
    HWI2C_Enable();
}

void _I2C_Disable(void)
{
    HWI2C_Disable();
    I2C_WAIT = 1L;
}

/*
        Export Functions - 1
*/
int I2C_ReadByte(int DeviceID,char *Buffer)
{
    int         Retry;

    _I2C_Initial();
    for (Retry=1;Retry<=READ_MAX_RETRIES;Retry++)
    {
        I2C_WAIT = Retry;
        _I2C_START();
        if (!_I2C_WriteData(DeviceID|0x01))
        {
#ifdef DEBUG
 printf("No slave address acknowledge %2d times - I2C_ReadByte()                    \n",Retry);
#endif
            _I2C_STOP();
            continue;
        }
        *Buffer = _I2C_ReadData() & 0xFF;
        _I2C_STOP();
        if (HWI2C_IsErrorOccur())
        {
#ifdef DEBUG
               printf("HWI2C_IsErrorOccur.                               \n");
#endif
               _I2C_Disable();
               return 0;
        }
        _I2C_Disable();
        return 1;
    }

    _I2C_Disable();
    return 0;
}

/*
        Export Functions - 2
*/
int I2C_Read(int DeviceID,int SubAddress,char *Buffer,int BufferLen)
{
    int         idx,Retry;

    _I2C_Initial();

    for (Retry=1;Retry<=READ_MAX_RETRIES_1;Retry++)
    {
        I2C_WAIT = Retry;
        _I2C_START();
        if (!_I2C_WriteData(DeviceID&0xFE))
        {
#ifdef DEBUG
 printf("No slave address acknowledge %2d times. I2C_Read() - 1   \n",Retry);
#endif
            _I2C_STOP();
            continue;
        }
        if (!_I2C_WriteData(SubAddress))
        {
#ifdef DEBUG
  printf("No subaddress acknowledge %2d times.                        \n",Retry);
#endif
            _I2C_STOP();
            continue;
        }
        _I2C_STOP();
        break;
    }

    for (Retry=1;Retry<=READ_MAX_RETRIES_2;Retry++)
    {
        I2C_WAIT = Retry;
        _I2C_START();
        if (!_I2C_WriteData(DeviceID|0x01))
        {
#ifdef DEBUG
 printf("No slave address acknowledge %2d times. I2C_Read() - 2     \n",Retry);
#endif
            _I2C_STOP();
            continue;
        }
        for (idx=0;idx<BufferLen;idx++)
        {
            *Buffer = (char)(_I2C_ReadData() & 0xFF);
            Buffer ++;
            if (HWI2C_IsErrorOccur())
            {
#ifdef DEBUG
               printf("HWI2C_IsErrorOccur.                               \n");
#endif
               _I2C_STOP();
               _I2C_Disable();
               return 0;
            }
        }
        _I2C_STOP();
        _I2C_Disable();
        return 1;
    }

    _I2C_STOP();
    _I2C_Disable();
    return 0;
}


/*
        Export Functions - 3
*/
int I2C_Write(int DeviceID,int SubAddress,int  Data)
{
    int Retry,Done=0;

    _I2C_Initial();
    for (Retry=1;Retry<=WRITE_MAX_RETRIES;Retry++)
    {
      I2C_WAIT = Retry;

      _I2C_START();

      if (!_I2C_WriteData(DeviceID))
      {
#ifdef DEBUG
  printf("No slave address acknowledge %2d times - I2C_Write() \n",Retry);
#endif
          _I2C_STOP();
          continue;
      }
      if (!_I2C_WriteData(SubAddress))
      {
#ifdef DEBUG
        printf("No subaddress acknowledge.                          \n");
#endif
          _I2C_STOP();
          continue;
      }
      if (!_I2C_WriteData(Data))
      {
#ifdef DEBUG
        printf("No data acknowledge.                                \n");
#endif
          _I2C_STOP();
          continue;
      }
      Done = 1;
      break;
    }

    _I2C_STOP();

    _I2C_Disable();
    return Done;
}

/*
        Export Functions - 4
*/
int I2C_WriteByte(int DeviceID,int SubAddress,int *Buffer,int BufferLen)
{
#if 1
// Byte Write Mode
   int idx;

   for (idx=0;idx<BufferLen;idx++)
   {
       if (!I2C_Write(DeviceID,SubAddress,*Buffer))
       {
           return 0;
       }
       Buffer ++;
       SubAddress ++;
   }
   return 1;
#else
// Sequence Write Mode - TBD

#endif
}

