#include <dos.h>
#include <stdio.h>

/*

*/
#include "i2c.c"

/*

*/
get_pci_busDeviceNo()
{
    union REGS r1;
    int   idx;

        r1.w.ax = 0xb102;
        r1.w.cx = 0x036E; // BT878
        r1.w.dx = 0x109E; // Conexant
        r1.w.si = 0;
        int386(0x1a, &r1, &r1);

        if (r1.w.ax & 0xff00)
        {
        }
        else return(r1.w.bx & 0xffff);

    printf("Cannot find BT878 device.\n");
    return(-1);
}

PCI_write_config_word(short busDeviceNo, int reg, int value)
{
    union REGS r1;

    reg = reg*4;

    r1.w.ax = 0xb10c;
    r1.w.bx = busDeviceNo;
    r1.w.di = reg;
    r1.w.cx = value & 0xffff;
    int386(0x1a, &r1, &r1);
}

read_PCI_config_reg(short busDeviceNo,int reg)
{
    union REGS r1;
    unsigned config_reg;

    reg = reg*4;

    r1.w.ax = 0xb109;               // Read Config Word
    r1.w.bx = busDeviceNo;
    r1.w.di = reg;          // address lsw.
    int386(0x1a, &r1, &r1);

    if (r1.w.ax & 0xff00) {
        printf("Reading PCI config lsw failed\n");
        return(-1);
    }
    config_reg = r1.w.cx;
    r1.w.ax = 0xb109;               // Read Config Word
    r1.w.bx = busDeviceNo;
    r1.w.di = reg+2;                // address msw.
    int386(0x1a, &r1, &r1);

    if (r1.w.ax & 0xff00) {
        printf("Reading PCI config msw failed\n");
        return(-1);
    }
    config_reg = (r1.w.cx << 16) | (config_reg & 0xffff);
    return(config_reg);
}


/*


*/
#define DEVICE_EEPROM 0xA0

void* mmap(void* addr, int len, int prot, int flags, int fd, int off)
{
  union REGS r1;

  /* Map Physical Address */
  r1.w.ax = 0x0800;
  r1.w.bx = ((int)addr >> 16) & 0xFFFF;
  r1.w.cx = ((int)addr >>  0) & 0xFFF0;
  r1.w.si = (len >> 16) & 0xffff;
  r1.w.di = (len >>  0) & 0xffff;
  int386(0x31, &r1, &r1);
  if (r1.x.cflag)  {
    /* errno = EINVAL; */
    return((void*)-1);
    printf("mapping physical address failed\n");
  }
  addr = (void*)((r1.w.bx << 16) + r1.w.cx);
  return(addr);
}

mapBT878() {
    short busDeviceNo;
    void * BT878_BaseAddr;

    // Get PCI Bus number and device number.
    busDeviceNo = get_pci_busDeviceNo();
    if (busDeviceNo == -1) {
        return(-1);
    }

#if 0
    // Make sure NV PCI memory access enabled.
    PCI_write_config_word( busDeviceNo, 1,
        read_PCI_config_reg( busDeviceNo, 1 ) | 0x0002 // Memory Access Enabled.
            );
#endif

    // Get the PCI device memory map physical starting address. 4K size
    BT878_BaseAddr = mmap((void*)read_PCI_config_reg(busDeviceNo,4),0x00001000,0,0,0,0);

    if (BT878_BaseAddr == (int *)-1)  {
        printf("Cannot mmmap(BT878)\n");
        return(-1);
    }
    BT878_I2CAddr = (int *)((long)BT878_BaseAddr+0x110);
    return(0);
} /* end mapBT878V */

int main(int argc,char *argv[])
{
    int Status,SubVendor,SubDevice,Data;
    int DataFC,DataFD,DataFE,DataFF;
    int ID1,ID2;
    long IDs;

    DataFC=DataFD=DataFE=DataFF=0;

    if (mapBT878()==-1) return 1;

    Data = 0xFF;
    Status = I2C_ReadByte(DEVICE_EEPROM,(char *)&Data);
    if (!Status)
    {
        printf("EEPROM not found.\n");
        return 1;
    }
    {
        Status = I2C_Read(DEVICE_EEPROM,0xFC,(char *)&IDs,4);
        if (!Status) return 1;
        DataFC = (int)(IDs & 0xFF);
        DataFD = (int)((IDs>>8) & 0xFF);
        DataFE = (int)((IDs>>16) & 0xFF);
        DataFF = (int)((IDs>>24) & 0xFF);
        ID1 = DataFF | (DataFE << 8);
        ID2 = DataFD | (DataFC << 8);
        printf("SubVendor ID : %4.4Xh, SubDevice ID: %4.4Xh.\n",ID1,ID2);
    }
    {
        int idx,eeprom_data;
        for (idx=0;idx<256;idx++)
        {
           eeprom_data = 0;
           Status = I2C_Read(DEVICE_EEPROM,idx,(char *)&eeprom_data,1);
           if (!Status) return 1;

           if (idx==0xFC) printf("%2.2X ",DataFC);
           else printf("%2.2X ",eeprom_data);

           if ( (idx&0x0F) == 0x0F) printf("\n");
        }
    }
   return 0;
}
